@@ -30,6 +30,14 @@
 #include <math.h>
 #include <stdlib.h>
 
+typedef enum
+{
+  META_LEFT,
+  META_RIGHT,
+  META_TOP,
+  META_BOTTOM
+} MetaWindowDirection;
+
 static gint
 northwestcmp (gconstpointer a, gconstpointer b)
 {
@@ -224,6 +232,159 @@ find_next_cascade (MetaWindow *window,
     }
 }
 
+static void
+find_most_freespace (MetaWindow *window,
+                     MetaFrameGeometry *fgeom,
+                     /* visible windows on relevant workspaces */
+                     MetaWindow *focus_window,
+                     int         x,
+                     int         y,
+                     int        *new_x,
+                     int        *new_y)
+{
+  MetaWindowDirection side;
+  int max_area;
+  int max_width, max_height, left, right, top, bottom;
+  int left_space, right_space, top_space, bottom_space;
+  int frame_size_left, frame_size_top;
+  MetaRectangle work_area;
+  MetaRectangle avoid;
+  MetaRectangle outer;
+
+  frame_size_left = fgeom ? fgeom->left_width : 0;
+  frame_size_top  = fgeom ? fgeom->top_height : 0;
+
+  meta_window_get_work_area_current_xinerama (focus_window, &work_area);
+  meta_window_get_outer_rect (focus_window, &avoid);
+  meta_window_get_outer_rect (window, &outer);
+
+  /* Find the areas of choosing the various sides of the focus window */
+  max_width  = MIN (avoid.width, outer.width);
+  max_height = MIN (avoid.height, outer.height);
+  left_space   = avoid.x - work_area.x;
+  right_space  = work_area.width - (avoid.x + avoid.width - work_area.x);
+  top_space    = avoid.y - work_area.y;
+  bottom_space = work_area.height - (avoid.y + avoid.height - work_area.y);
+  left   = MIN (left_space,   outer.width);
+  right  = MIN (right_space,  outer.width);
+  top    = MIN (top_space,    outer.height);
+  bottom = MIN (bottom_space, outer.height);
+
+  /* Find out which side of the focus_window can show the most of the window */
+  side = META_LEFT;
+  max_area = left*max_height;
+  if (right*max_height > max_area)
+    {
+      side = META_RIGHT;
+      max_area = right*max_height;
+    }
+  if (top*max_width > max_area)
+    {
+      side = META_TOP;
+      max_area = top*max_width;
+    }
+  if (bottom*max_width > max_area)
+    {
+      side = META_BOTTOM;
+      max_area = bottom*max_width;
+    }
+
+  /* Give up if there's no where to put it (i.e. focus window is maximized) */
+  if (max_area == 0)
+    return;
+
+  /* Place the window on the relevant side; if the whole window fits,
+   * make it adjacent to the focus window; if not, make sure the
+   * window doesn't go off the edge of the screen.
+   */
+  switch (side)
+    {
+    case META_LEFT:
+      *new_y = avoid.y + frame_size_top;
+      if (left_space > outer.width)
+        *new_x = avoid.x - outer.width + frame_size_left;
+      else
+        *new_x = work_area.x + frame_size_left;
+      break;
+    case META_RIGHT:
+      *new_y = avoid.y + frame_size_top;
+      if (right_space > outer.width)
+        *new_x = avoid.x + avoid.width + frame_size_left;
+      else
+        *new_x = work_area.x + work_area.width - outer.width + frame_size_left;
+      break;
+    case META_TOP:
+      *new_x = avoid.x + frame_size_left;
+      if (top_space > outer.height)
+        *new_y = avoid.y - outer.height + frame_size_top;
+      else
+        *new_y = work_area.y + frame_size_top;
+      break;
+    case META_BOTTOM:
+      *new_x = avoid.x + frame_size_left;
+      if (bottom_space > outer.height)
+        *new_y = avoid.y + avoid.height + frame_size_top;
+      else
+        *new_y = work_area.y + work_area.height - outer.height + frame_size_top;
+      break;
+    }
+}
+
+static void
+avoid_being_obscured_as_second_modal_dialog (MetaWindow *window,
+                                             MetaFrameGeometry *fgeom,
+                                             int        *x,
+                                             int        *y)
+{
+  /* We can't center this dialog if it was denied focus and it
+   * overlaps with the focus window and this dialog is modal and this
+   * dialog is in the same app as the focus window (*phew*...please
+   * don't make me say that ten times fast). See bug 307875 comment 11
+   * and 12 for details, but basically it means this is probably a
+   * second modal dialog for some app while the focus window is the
+   * first modal dialog.  We should probably make them simultaneously
+   * visible in general, but it becomes mandatory to do so due to
+   * buggy apps (e.g. those using gtk+ *sigh*) because in those cases
+   * this second modal dialog also happens to be modal to the first
+   * dialog in addition to the main window, while it has only let us
+   * know about the modal-to-the-main-window part.
+   */
+
+  MetaWindow *focus_window;
+  MetaRectangle overlap;
+
+  focus_window = window->display->focus_window;
+
+  if (window->denied_focus_and_not_transient &&
+      window->wm_state_modal && /* FIXME: Maybe do this for all transients? */
+      meta_window_same_application (window, focus_window) &&
+      meta_rectangle_intersect (&window->rect,
+                                &focus_window->rect,
+                                &overlap))
+    {
+      find_most_freespace (window, fgeom, focus_window, *x, *y, x, y);
+      meta_topic (META_DEBUG_PLACEMENT,
+                  "Dialog window %s was denied focus but may be modal "
+                  "to the focus window; had to move it to avoid the "
+                  "focus window\n",
+                  window->desc);
+    }
+
+  meta_topic (META_DEBUG_PLACEMENT,
+              "Status:\n"
+              "  denied focus: %d\n"
+              "  modal:        %d\n"
+              "  same app:     %d\n"
+              "  overlaps      %d\n",
+              window->denied_focus_and_not_transient,
+              window->wm_state_modal,
+              meta_window_same_application (window, focus_window),
+              meta_rectangle_intersect (&window->rect,
+                                        &focus_window->rect,
+                                        &overlap)
+              );
+}
+
 static int
 intcmp (const void* a, const void* b)
 {
@@ -649,6 +810,7 @@ meta_window_place (MetaWindow        *window,
         {
           meta_topic (META_DEBUG_PLACEMENT,
                       "Not placing window with PPosition or USPosition set\n");
+          avoid_being_obscured_as_second_modal_dialog (window, fgeom, &x, &y);
           goto done_no_constraints;
         }
     }
@@ -700,6 +862,8 @@ meta_window_place (MetaWindow        *window,
           meta_topic (META_DEBUG_PLACEMENT, "Centered window %s over transient parent\n",
                       window->desc);
           
+          avoid_being_obscured_as_second_modal_dialog (window, fgeom, &x, &y);
+
           goto done;
         }
     }
@@ -748,13 +912,16 @@ meta_window_place (MetaWindow        *window,
       {
         MetaWindow *w = tmp->data;
 
-        if (!w->minimized &&
+        if (meta_window_showing_on_its_workspace (w) &&
             w != window && 
-            meta_window_shares_some_workspace (window, w))
+            (window->workspace == w->workspace ||
+             window->on_all_workspaces || w->on_all_workspaces))
           windows = g_list_prepend (windows, w);
 
         tmp = tmp->next;
       }
+
+    g_slist_free (all_windows);
   }
 
   /* Warning, this is a round trip! */
@@ -771,7 +938,7 @@ meta_window_place (MetaWindow        *window,
   if (find_first_fit (window, fgeom, windows,
                       xineramas_list, n_xineramas,
                       x, y, &x, &y))
-    goto done;
+    goto done_check_denied_focus;
 
   /* This is a special-case origin-cascade so that windows that are
    * too large to fit onto a workspace (and which will be
@@ -814,7 +981,7 @@ meta_window_place (MetaWindow        *window,
   if (placed_on == -1)
     {
       find_next_cascade (window, fgeom, windows, x, y, &x, &y);
-      placed_on = 0;
+      placed_on = window->screen->last_xinerama_index;
     }
 
   /* Maximize windows if they are too big for their work area (bit of
@@ -838,6 +1005,51 @@ meta_window_place (MetaWindow        *window,
           window->maximize_after_placement = TRUE;
         }
     }
+
+ done_check_denied_focus:
+  /* If the window is being denied focus and isn't a transient of the
+   * focus window, we do NOT want it to overlap with the focus window
+   * if at all possible.  This is guaranteed to only be called if the
+   * focus_window is non-NULL, and we try to avoid that window.
+   */
+  if (window->denied_focus_and_not_transient)
+    {
+      gboolean       found_fit;
+      MetaWindow    *focus_window;
+      MetaRectangle  overlap;
+
+      focus_window = window->display->focus_window;
+      g_assert (focus_window != NULL);
+
+      /* No need to do anything if the window doesn't overlap at all */
+      found_fit = !meta_rectangle_intersect (&window->rect,
+                                             &focus_window->rect,
+                                             &overlap);
+
+      /* Try to do a first fit again, this time only taking into account the
+       * focus window.
+       */
+      if (!found_fit)
+        {
+          GList *focus_window_list;
+          focus_window_list = g_list_prepend (NULL, focus_window);
+
+          /* Reset x and y ("origin" placement algorithm) */
+          x = xi->x_origin;
+          y = xi->y_origin;
+
+          found_fit = find_first_fit (window, fgeom, focus_window_list,
+                                      xineramas_list, n_xineramas,
+                                      x, y, &x, &y);
+          g_list_free (focus_window_list);
+	}
+
+      /* If that still didn't work, just place it where we can see as much
+       * as possible.
+       */
+      if (!found_fit)
+        find_most_freespace (window, fgeom, focus_window, x, y, &x, &y);
+    }
   
  done:
   g_free (xineramas_list);
@@ -876,10 +1088,7 @@ get_windows_on_same_workspace (MetaWindow *window,
     {
       MetaWindow *w = tmp->data;
       
-      if (!w->minimized &&
-          w != window && 
-          meta_window_visible_on_workspace (w,
-                                            window->screen->active_workspace))
+      if (meta_window_should_be_showing (w) && w != window)
         {            
           windows = g_slist_prepend (windows, w);
           ++i;
@@ -891,6 +1100,8 @@ get_windows_on_same_workspace (MetaWindow *window,
   if (n_windows)
     *n_windows = i;
   
+  g_slist_free (all_windows);
+
   return windows;
 }
 
