@@ -41,12 +41,16 @@ typedef enum
   META_PREF_SCREEN_KEYBINDINGS,
   META_PREF_DISABLE_WORKAROUNDS,
   META_PREF_COMMANDS,
+  META_PREF_TERMINAL_COMMAND,
   META_PREF_BUTTON_LAYOUT,
   META_PREF_WORKSPACE_NAMES,
   META_PREF_VISUAL_BELL,
   META_PREF_AUDIBLE_BELL,
   META_PREF_VISUAL_BELL_TYPE,
-  META_PREF_REDUCED_RESOURCES
+  META_PREF_REDUCED_RESOURCES,
+  META_PREF_GNOME_ACCESSIBILITY,
+  META_PREF_CURSOR_THEME,
+  META_PREF_CURSOR_SIZE
 } MetaPreference;
 
 typedef void (* MetaPrefsChangedFunc) (MetaPreference pref,
@@ -71,11 +75,15 @@ gboolean                    meta_prefs_get_disable_workarounds (void);
 gboolean                    meta_prefs_get_auto_raise         (void);
 int                         meta_prefs_get_auto_raise_delay   (void);
 gboolean                    meta_prefs_get_reduced_resources  (void);
+gboolean                    meta_prefs_get_gnome_accessibility (void);
 
 const char*                 meta_prefs_get_command            (int i);
 
 char*                       meta_prefs_get_gconf_key_for_command (int i);
 
+const char*                 meta_prefs_get_terminal_command   (void);
+const char*                 meta_prefs_get_gconf_key_for_terminal_command (void);
+
 void                        meta_prefs_get_button_layout (MetaButtonLayout *button_layout);
 MetaActionDoubleClickTitlebar meta_prefs_get_action_double_click_titlebar (void);
 
@@ -85,6 +93,9 @@ const char* meta_prefs_get_workspace_name    (int         i);
 void        meta_prefs_change_workspace_name (int         i,
                                               const char *name);
 
+const char* meta_prefs_get_cursor_theme      (void);
+int         meta_prefs_get_cursor_size       (void);
+
 /* Screen bindings */
 #define META_KEYBINDING_WORKSPACE_1              "switch_to_workspace_1"
 #define META_KEYBINDING_WORKSPACE_2              "switch_to_workspace_2"
@@ -147,6 +158,7 @@ void        meta_prefs_change_workspace_name (int         i,
 #define META_KEYBINDING_COMMAND_32               "run_command_32"
 #define META_KEYBINDING_COMMAND_SCREENSHOT       "run_command_screenshot"
 #define META_KEYBINDING_COMMAND_WIN_SCREENSHOT   "run_command_window_screenshot"
+#define META_KEYBINDING_RUN_COMMAND_TERMINAL     "run_command_terminal"
 
 /* Window bindings */
 #define META_KEYBINDING_WINDOW_MENU              "activate_window_menu"
